#!/usr/bin/env python
# -*- coding: utf-8 -*-
import logging
import os
import argparse
from typing import Optional

import click
import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt
from pathlib import Path
from deprecated import deprecated

from GANDLF.cli import copyrightMessage
from GANDLF.entrypoints import append_copyright_to_help
from GANDLF.utils import logger_setup


def plot_all(df_training, df_validation, df_testing, output_plot_dir):
    """
    Plots training, validation, and testing data for loss and other metrics.
    TODO: this function needs to be moved under utils and then called after every training epoch.

    Args:
        df_training (pd.DataFrame): DataFrame containing training data.
        df_validation (pd.DataFrame): DataFrame containing validation data.
        df_testing (pd.DataFrame): DataFrame containing testing data.
        output_plot_dir (str): Directory to save the plots.

    Returns:
        tuple: Tuple containing the modified training, validation, and testing DataFrames.
    """
    # Drop any columns that might have "_" in the values of their rows
    banned_cols = [
        col
        for col in df_training.columns
        if any("_" in str(val) for val in df_training[col].values)
    ]

    # Determine metrics from the column names by removing the "train_" prefix
    metrics = [
        col.replace("train_", "")
        for col in df_training.columns
        if "train_" in col and col not in banned_cols
    ]

    # Split the values of the banned columns into multiple columns
    # for df in [df_training, df_validation, df_testing]:
    #     for col in banned_cols:
    #         if df[col].dtype == "object":
    #             split_cols = (
    #                 df[col]
    #                 .str.split("_", expand=True)
    #                 .apply(pd.to_numeric, errors="coerce")
    #             )
    #             split_cols.columns = [f"{col}_{i}" for i in range(split_cols.shape[1])]
    #             df.drop(columns=col, inplace=True)
    #             df = pd.concat([df, split_cols], axis=1)

    # Check if any of the metrics is present in the column names of the dataframe
    assert any(
        any(metric in col for col in df_training.columns) for metric in metrics
    ), "None of the specified metrics is in the dataframe."

    required_cols = ["epoch_no", "train_loss"]

    # Check if the required columns are in the dataframe
    assert all(
        col in df_training.columns for col in required_cols
    ), "Not all required columns are in the dataframe."

    epochs = len(df_training)

    # Plot for loss
    plt.figure(figsize=(12, 6))
    if "train_loss" in df_training.columns:
        sns.lineplot(data=df_training, x="epoch_no", y="train_loss", label="Training")

    if "valid_loss" in df_validation.columns:
        sns.lineplot(
            data=df_validation, x="epoch_no", y="valid_loss", label="Validation"
        )

    if df_testing is not None and "test_loss" in df_testing.columns:
        sns.lineplot(data=df_testing, x="epoch_no", y="test_loss", label="Testing")

    plt.xlim(0, epochs - 1)
    plt.xlabel("Epoch")
    plt.ylabel("Loss")
    plt.title("Loss Plot")
    plt.legend()
    Path(output_plot_dir).mkdir(parents=True, exist_ok=True)
    plt.savefig(os.path.join(output_plot_dir, "loss_plot.png"), dpi=300)
    plt.close()

    # Plot for other metrics
    for metric in metrics:
        metric_cols = [col for col in df_training.columns if metric in col]
        for metric_col in metric_cols:
            plt.figure(figsize=(12, 6))
            if metric_col in df_training.columns:
                sns.lineplot(
                    data=df_training,
                    x="epoch_no",
                    y=metric_col,
                    label=f"Training {metric_col}",
                )
            if metric_col.replace("train", "valid") in df_validation.columns:
                sns.lineplot(
                    data=df_validation,
                    x="epoch_no",
                    y=metric_col.replace("train", "valid"),
                    label=f"Validation {metric_col}",
                )
            if (
                df_testing is not None
                and metric_col.replace("train", "test") in df_testing.columns
            ):
                sns.lineplot(
                    data=df_testing,
                    x="epoch_no",
                    y=metric_col.replace("train", "test"),
                    label=f"Testing {metric_col}",
                )
            plt.xlim(0, epochs - 1)
            plt.xlabel("Epoch")
            plt.ylabel(metric.capitalize())
            plt.title(f"{metric.capitalize()} Plot")
            plt.legend()
            plt.savefig(os.path.join(output_plot_dir, f"{metric}_plot.png"), dpi=300)
            plt.close()

    print("Plots saved successfully.")
    return df_training, df_validation, df_testing


def _read_data_and_plot(
    training_logs_path: str,
    validation_logs_path: str,
    testing_logs_path: Optional[str],
    output_plot_path: str,
    output_file: str,
):
    # moved out from _collect_stats for easier testing
    # Read all the files
    df_training = pd.read_csv(training_logs_path)
    df_validation = pd.read_csv(validation_logs_path)
    df_testing = pd.read_csv(testing_logs_path) if testing_logs_path else None

    # Check for metrics in columns and do tight plots
    plot_all(df_training, df_validation, df_testing, output_plot_path)

    df_training["split"] = "train"
    df_testing["split"] = "test"
    df_validation["split"] = "validation"
    pd.concat((df_training, df_testing, df_validation)).to_csv(output_file)


def _collect_stats(model_dir: str, output_dir: str):
    input_dir = os.path.normpath(model_dir)
    output_dir = os.path.normpath(output_dir)
    Path(output_dir).mkdir(parents=True, exist_ok=True)
    output_file = os.path.join(output_dir, "data.csv")  # data file name
    output_plot = os.path.join(output_dir, "plot.png")  # plot file

    training_logs = os.path.join(input_dir, "logs_training.csv")
    validation_logs = os.path.join(input_dir, "logs_validation.csv")
    testing_logs = os.path.join(input_dir, "logs_testing.csv")
    if not os.path.isfile(testing_logs):
        logging.info(f"testing logs file was not found: {testing_logs}")
        testing_logs = None

    _read_data_and_plot(
        training_logs, validation_logs, testing_logs, output_plot, output_file
    )


@click.command()
@click.option(
    "--model-dir",
    "-m",
    type=click.Path(exists=True, file_okay=False, dir_okay=True),
    required=True,
    help="Input directory which contains testing and validation models log files",
)
@click.option(
    "--output-dir",
    "-o",
    type=click.Path(file_okay=False, dir_okay=True),
    required=True,
    help="Output directory to save stats and plot",
)
@click.option(
    "--log-file",
    type=click.Path(),
    default=None,
    help="Output file which will contain the logs.",
)
@append_copyright_to_help
def new_way(model_dir: str, output_dir: str, log_file: str):
    """Collect statistics from different testing/validation combinations from output directory."""
    logger_setup(log_file)
    _collect_stats(model_dir=model_dir, output_dir=output_dir)


@deprecated(
    "This is a deprecated way of running GanDLF. Please, use `gandlf collect-stats` cli command "
    + "instead of `gandlf_collectStats`. Note that in new CLI tool params were renamed to snake-case:\n"
    + "  --modeldir to --model-dir\n"
    + "  --outputdir to --output-dir\n"
    + "`gandlf_collectStats` script would be deprecated soon."
)
def old_way():
    logger_setup()
    parser = argparse.ArgumentParser(
        prog="GANDLF_CollectStats",
        formatter_class=argparse.RawTextHelpFormatter,
        description="Collect statistics from different testing/validation combinations from output directory.\n\n"
        + copyrightMessage,
    )
    parser.add_argument(
        "-m",
        "--modeldir",
        metavar="",
        type=str,
        help="Input directory which contains testing and validation models log files",
    )
    parser.add_argument(
        "-o",
        "--outputdir",
        metavar="",
        type=str,
        help="Output directory to save stats and plot",
    )

    args = parser.parse_args()
    _collect_stats(args.modeldir, args.outputdir)


if __name__ == "__main__":
    old_way()



from .modelBase import ModelBase
import monai.networks.nets.dynunet as dynunet


class dynunet_wrapper(ModelBase):
    """
    More info: https://docs.monai.io/en/stable/networks.html#dynunet

    Args:
        spatial_dims (int): number of spatial dimensions.
        in_channels (int): number of input channels.
        out_channels (int): number of output channels.
        kernel_size (Sequence[Union[Sequence[int], int]]): convolution kernel size.
        strides (Sequence[Union[Sequence[int], int]]): convolution strides for each blocks.
        upsample_kernel_size (Sequence[Union[Sequence[int], int]]): convolution kernel size for transposed convolution layers. The values should equal to strides[1:].
        filters (Optional[Sequence[int]]): number of output channels for each blocks.  Defaults to None.
        dropout (Union[Tuple, str, float, None]): dropout ratio. Defaults to no dropout.
        norm_name (Union[Tuple, str]): feature normalization type and arguments. Defaults to INSTANCE.
        act_name (Union[Tuple, str]): activation layer type and arguments. Defaults to leakyrelu.
        deep_supervision (bool): whether to add deep supervision head before output. Defaults to False.
        deep_supr_num (int): number of feature maps that will output during deep supervision head. The value should be larger than 0 and less than the number of up sample layers. Defaults to 1.
        res_block (bool): whether to use residual connection based convolution blocks during the network. Defaults to False.
        trans_bias (bool): whether to set the bias parameter in transposed convolution layers. Defaults to False.
    """

    def __init__(self, parameters: dict):
        super(dynunet_wrapper, self).__init__(parameters)

        # checking for validation
        assert (
            "kernel_size" in parameters["model"]
        ) == True, "\033[0;31m`kernel_size` key missing in parameters"
        assert (
            "strides" in parameters["model"]
        ) == True, "\033[0;31m`strides` key missing in parameters"

        # defining some defaults
        # if not ("upsample_kernel_size" in parameters["model"]):
        #     parameters["model"]["upsample_kernel_size"] = parameters["model"][
        #         "strides"
        #     ][1:]

        parameters["model"]["filters"] = parameters["model"].get("filters", None)
        parameters["model"]["act_name"] = parameters["model"].get(
            "act_name", ("leakyrelu", {"inplace": True, "negative_slope": 0.01})
        )

        parameters["model"]["deep_supervision"] = parameters["model"].get(
            "deep_supervision", True
        )

        parameters["model"]["deep_supr_num"] = parameters["model"].get(
            "deep_supr_num", 1
        )

        parameters["model"]["res_block"] = parameters["model"].get("res_block", True)

        parameters["model"]["trans_bias"] = parameters["model"].get("trans_bias", False)
        parameters["model"]["dropout"] = parameters["model"].get("dropout", None)

        if not ("norm_type" in parameters["model"]):
            self.norm_type = "INSTANCE"

        self.model = dynunet.DynUNet(
            spatial_dims=self.n_dimensions,
            in_channels=self.n_channels,
            out_channels=self.n_classes,
            kernel_size=parameters["model"]["kernel_size"],
            strides=parameters["model"]["strides"],
            upsample_kernel_size=parameters["model"]["strides"][1:],
            filters=parameters["model"][
                "filters"
            ],  # number of output channels for each blocks
            dropout=parameters["model"][
                "dropout"
            ],  # dropout ratio. Defaults to no dropout
            norm_name=self.norm_type,
            act_name=parameters["model"]["act_name"],
            deep_supervision=parameters["model"]["deep_supervision"],
            deep_supr_num=parameters["model"][
                "deep_supr_num"
            ],  # number of feature maps that will output during deep supervision head.
            res_block=parameters["model"]["res_block"],
            trans_bias=parameters["model"]["trans_bias"],
        )

    def forward(self, x):
        return self.model.forward(x)



from monai.optimizers import WarmupCosineSchedule as WCS


def warmupcosineschedule(parameters):
    parameters["scheduler"]["warmup_steps"] = parameters["scheduler"].get(
        "warmup_steps", 0.1 * parameters["num_epochs"]
    )

    return WCS(
        parameters["optimizer_object"],
        t_total=parameters["num_epochs"],
        warmup_steps=parameters["scheduler"]["warmup_steps"],
    )



# -*- coding: utf-8 -*-
# adapted from https://github.com/qubvel/segmentation_models.pytorch
from typing import Optional, Union, List
import torch
import torch.nn as nn

from segmentation_models_pytorch.base import SegmentationHead, ClassificationHead
from segmentation_models_pytorch.encoders import get_encoder
from segmentation_models_pytorch.decoders.unet.decoder import UnetDecoder
from segmentation_models_pytorch.base import initialization as init

from .modelBase import ModelBase


class SegmentationModel(torch.nn.Module):
    """
    This class defines a segmentation model. It takes an input tensor and sequentially passes it through the model's
    encoder, decoder and heads.

    Methods:
        initialize:
            Initializes the model's classification or segmentation head

        forward:
            Sequentially passes the input tensor through the model's encoder, decoder, and heads and returns either the
            segmentation masks or the class labels.
    """

    def initialize(self):
        """
        This function initializes the model's classification or segmentation head.
        """
        if self.classification_head is None:
            init.initialize_decoder(self.decoder)
            init.initialize_head(self.segmentation_head)
        else:
            init.initialize_head(self.classification_head)

    def forward(self, x):
        """
        This function sequentially passes the input tensor through the model's encoder, decoder, and heads and returns
        either the segmentation masks or the class labels.

        Args:
            x (torch.Tensor): Input tensor

        Returns:
            torch.Tensor: Segmentation masks or class labels
        """
        features = self.encoder(x)

        if self.classification_head is None:
            decoder_output = self.decoder(*features)
            masks = self.segmentation_head(decoder_output)
            return masks
        else:
            labels = self.classification_head(features[-1])
            return labels

    ## commented out because we are not using this interface
    # def predict(self, x):
    #     """Inference method. Switch model to `eval` mode, call `.forward(x)` with `torch.no_grad()`

    #     Args:
    #         x: 4D torch tensor with shape (batch_size, channels, height, width)

    #     Return:
    #         prediction: 4D torch tensor with shape (batch_size, classes, height, width)

    #     """
    #     if self.training:
    #         self.eval()

    #     with torch.no_grad():
    #         x = self.forward(x)

    #     return x


class Unet(SegmentationModel):
    """
    This has been adapted from its original implementation in
    https://github.com/qubvel/segmentation_models.pytorch/blob/master/segmentation_models_pytorch/decoders/unet/model.py
    Unet_ is a fully convolution neural network for image semantic segmentation. Consist of *encoder*
    and *decoder* parts connected with *skip connections*. Encoder extract features of different spatial
    resolution (skip connections) which are used by decoder to define accurate segmentation mask. Use *concatenation*
    for fusing decoder blocks with skip connections.

    Args:
        encoder_name (str): Name of the classification model to be used as an encoder (a.k.a backbone) to extract features of different spatial resolution.
        encoder_depth (int): A number of stages used in encoder in range [3, 5]. Each stage generates features two times smaller in spatial dimensions than the previous one. Default is 5.
        encoder_weights (str or None): One of "None" (random initialization), "imagenet" (pre-training on ImageNet) and other pretrained weights (see table with available weights for each encoder_name).
        decoder_channels (list of ints): List of integers specifying in_channels parameter for convolutions used in decoder. Length of the list should be the same as encoder_depth.
        decoder_use_batchnorm (bool or str): If True, BatchNorm2d layer between Conv2D and Activation layers is used. If "inplace", InplaceABN will be used, allowing to decrease memory consumption. Available options are True, False, "inplace".
        decoder_attention_type (str or None): Attention module used in the decoder of the model. Available options are None and "scse". SCSE paper - https://arxiv.org/abs/1808.08127.
        in_channels (int): A number of input channels for the model. Default is 3 (RGB images).
        classes (int): A number of classes for output mask (or you can think as a number of channels of output mask).
        activation (str, callable, or None): An activation function to apply after the final convolution layer. Available options are "sigmoid", "softmax", "logsoftmax", "tanh", "identity", callable, and None. Default is None.
        aux_params (dict or None): Dictionary with parameters of the auxiliary output (classification head). Auxiliary output is built on top of the encoder if aux_params is not None (default). Supported params:
            - classes (int): A number of classes.
            - pooling (str): One of "max", "avg". Default is "avg".
            - dropout (float): Dropout factor in [0, 1).
            - activation (str): An activation function to apply "sigmoid"/"softmax" (could be None to return logits).

    Returns:
        model (torch.nn.Module): A UNet model for image semantic segmentation.

    Reference:
    Ronneberger, O., Fischer, P., & Brox, T. (2015). U-Net: Convolutional Networks for Biomedical Image Segmentation.
    arXiv preprint arXiv:1505.04597.

    """

    def __init__(
        self,
        encoder_name: str = "resnet34",
        encoder_depth: int = 5,
        encoder_weights: Optional[str] = "imagenet",
        decoder_use_batchnorm: bool = True,
        decoder_channels: List[int] = (256, 128, 64, 32, 16),
        decoder_attention_type: Optional[str] = None,
        in_channels: int = 3,
        classes: int = 1,
        activation: Optional[Union[str, callable]] = None,
        aux_params: Optional[dict] = None,
    ):
        super().__init__()

        def _get_fixed_encoder_channels(name, in_channels):
            if "mit_" in name:
                # MixVision Transformers only support 3-channel inputs
                return 3
            else:
                return in_channels

        self.encoder = get_encoder(
            encoder_name,
            in_channels=_get_fixed_encoder_channels(encoder_name, in_channels),
            depth=encoder_depth,
            weights=encoder_weights,
        )
        out_channels = self.encoder.out_channels
        if "mit_" in encoder_name:
            # MixVision Transformers only support 3-channel inputs
            print(
                "WARNING: MixVision Transformers only support 3 channels, adding an extra Conv layer for compatibility."
            )
            self.pre_encoder = nn.Conv2d(in_channels, 3, kernel_size=1, stride=1)
            modules = []
            modules.append(self.pre_encoder)
            modules.append(self.encoder)
            self.encoder = nn.Sequential(*modules)

        if aux_params is None:
            self.decoder = UnetDecoder(
                encoder_channels=out_channels,
                decoder_channels=decoder_channels,
                n_blocks=encoder_depth,
                use_batchnorm=decoder_use_batchnorm,
                center=True if encoder_name.startswith("vgg") else False,
                attention_type=decoder_attention_type,
            )
            self.segmentation_head = SegmentationHead(
                in_channels=decoder_channels[-1],
                out_channels=classes,
                activation=activation,
                kernel_size=3,
            )
            self.classification_head = None
        else:
            self.classification_head = ClassificationHead(
                in_channels=out_channels[-1], **aux_params
            )
            self.segmentation_head = None

        self.name = "u-{}".format(encoder_name)
        self.initialize()


class ImageNet_UNet(ModelBase):
    """
    Wrapper class to parse arguments from configuration and construct appropriate model object.

    Args:
        ModelBase (nn.Module): The base model class.
    """

    def __init__(self, parameters) -> None:
        super(ImageNet_UNet, self).__init__(parameters)

        # https://github.com/qubvel/segmentation_models.pytorch/issues/745
        import ssl

        ssl._create_default_https_context = ssl._create_unverified_context

        decoder_use_batchnorm = False
        if parameters["model"]["norm_type"] == "batch":
            decoder_use_batchnorm = True
        decoder_use_batchnorm = parameters["model"].get(
            "decoder_use_batchnorm", decoder_use_batchnorm
        )

        default_encoder_name = parameters["model"].get("encoder_name", "resnet34")
        # MixVision Transformers only support 2D inputs with 3 channels (the channel dimension is checked in the encoder)
        if ("mit_" in default_encoder_name) or ("timm-" in default_encoder_name):
            assert (
                self.n_dimensions == 2
            ), "MixVision Transformers and TIMM models only support 2D inputs"

        encoder_depth = parameters["model"].get("depth", 5)
        encoder_depth = parameters["model"].get("encoder_depth", encoder_depth)
        parameters["model"]["pretrained"] = parameters["model"].get("pretrained", True)
        if parameters["model"]["pretrained"]:
            parameters["model"]["encoder_weights"] = "imagenet"
            # special case for some encoders: https://github.com/search?q=repo%3Aqubvel%2Fsegmentation_models.pytorch%20imagenet%2B5k&type=code
            if default_encoder_name in ["dpn68b", "dpn92", "dpn107"]:
                parameters["model"]["encoder_weights"] = "imagenet+5k"
        else:
            parameters["model"]["encoder_weights"] = None

        if parameters["problem_type"] != "segmentation":
            classifier_head_parameters = {}
            classifier_head_parameters["classes"] = self.n_classes
            classifier_head_parameters["activation"] = parameters["model"][
                "final_layer"
            ]
            if classifier_head_parameters["activation"] == "None":
                classifier_head_parameters["activation"] = None
            classifier_head_parameters["dropout"] = parameters["model"].get(
                "dropout", 0.2
            )
            classifier_head_parameters["pooling"] = parameters["model"].get(
                "pooling", "avg"
            )
        else:
            classifier_head_parameters = None

        self.model = Unet(
            encoder_name=default_encoder_name,
            encoder_weights=parameters["model"]["encoder_weights"],
            in_channels=self.n_channels,
            classes=self.n_classes,
            activation=parameters["model"]["final_layer"],
            decoder_use_batchnorm=decoder_use_batchnorm,
            decoder_attention_type=parameters["model"].get(
                "decoder_attention_type", None
            ),
            encoder_depth=encoder_depth,
            decoder_channels=parameters["model"].get(
                "decoder_channels", (256, 128, 64, 32, 16)
            ),
            aux_params=classifier_head_parameters,
        )

        if self.n_dimensions == 3:
            self.model = self.converter(self.model).model

    def forward(self, x):
        """
        Forward pass of the model.

        Args:
            x (torch.Tensor): Input tensor.

        Returns:
            torch.Tensor: Output tensor.
        """
        return self.model(x)


def imagenet_unet_wrapper(parameters):
    return ImageNet_UNet(parameters)



#!usr/bin/env python
# -*- coding: utf-8 -*-
import logging
import os
import argparse
import yaml
from typing import Optional
import click
from deprecated import deprecated

from GANDLF.anonymize import run_anonymizer
from GANDLF.cli import copyrightMessage
from GANDLF.entrypoints import append_copyright_to_help
from GANDLF.utils.gandlf_logging import logger_setup


def _anonymize_images(
    input_dir: str, output_file: str, config_path: Optional[str], modality: str
):
    input_dir = os.path.normpath(input_dir)
    output_file = os.path.normpath(output_file)
    # TODO: raise an error if config pass provided but not exist (user made a typo?)
    config = None
    if config_path and os.path.isfile(config_path):
        config = yaml.safe_load(open(config_path, "r"))

    logging.debug(f"{input_dir=}")
    logging.debug(f"{output_file=}")
    logging.debug(f"{config=}")
    logging.debug(f"{modality=}")
    run_anonymizer(input_dir, output_file, config, modality)

    logging.info("Finished successfully.")


# new way of defining params via click
@click.command()
@click.option(
    "--input-dir",
    "-i",
    required=True,
    type=click.Path(exists=True),
    help="Input directory or file which contains images to be anonymized.",
)
@click.option(
    "--config",
    "-c",
    help="Config (in YAML) for running anonymization, optionally, specify modality using '-m' for defaults.",
    type=click.Path(exists=True, file_okay=True, dir_okay=False),
)
@click.option(
    "--modality",
    "-m",
    default="rad",
    type=click.Choice(["rad", "histo"]),
    help="The modality type, can be 'rad' or 'histo'.",
)
@click.option(
    "--output-file",
    "-o",
    required=True,
    type=click.Path(),
    help="Output directory or file which will contain the image(s) after anonymization.",
)
@click.option(
    "--log-file",
    type=click.Path(),
    default=None,
    help="Output file which will contain the logs.",
)
@append_copyright_to_help
def new_way(input_dir, config, modality, output_file, log_file):
    """Anonymize images/scans in the data directory."""
    logger_setup(log_file)
    _anonymize_images(input_dir, output_file, config, modality)


# old-fashioned way of running gandlf via `gandlf_anonymizer`.
@deprecated(
    "This is a deprecated way of running GanDLF. Please, use `gandlf anonymizer` cli command "
    + "instead of `gandlf_anonymizer`. Note that in new CLI tool some params were renamed:\n"
    + "  --inputDir to --input-dir\n"
    + "  --outputFile to --output-file\n"
    + "`gandlf_anonymizer` script would be deprecated soon."
)
def old_way():
    logger_setup()
    parser = argparse.ArgumentParser(
        prog="GANDLF_Anonymize",
        formatter_class=argparse.RawTextHelpFormatter,
        description="Anonymize images/scans in the data directory.\n\n"
        + copyrightMessage,
    )
    parser.add_argument(
        "-i",
        "--inputDir",
        metavar="",
        type=str,
        help="Input directory or file which contains images to be anonymized.",
    )
    parser.add_argument(
        "-c",
        "--config",
        metavar="",
        default="",
        type=str,
        help="config (in YAML) for running anonymization, optionally, specify modality using '-m' for defaults.",
    )
    parser.add_argument(
        "-m",
        "--modality",
        metavar="",
        default="rad",
        type=str,
        help="The modality type, can be 'rad' or 'histo'.",
    )
    parser.add_argument(
        "-o",
        "--outputFile",
        metavar="",
        type=str,
        help="Output directory or file which will contain the image(s) after anonymization.",
    )
    args = parser.parse_args()

    # check for required parameters - this is needed here to keep the cli clean
    for param_name in ["inputDir", "outputFile"]:
        param_none_check = getattr(args, param_name)
        assert param_none_check is not None, f"Missing required parameter: {param_name}"

    inputDir = args.inputDir
    outputFile = args.outputFile
    config = args.config or None

    _anonymize_images(inputDir, outputFile, config, args.modality)


# main function
if __name__ == "__main__":
    old_way()



#!usr/bin/env python
# -*- coding: utf-8 -*-

import argparse
import os
import yaml
from typing import Optional

import click
from deprecated.classic import deprecated

from GANDLF.cli import copyrightMessage, split_data_and_save_csvs
from GANDLF.entrypoints import append_copyright_to_help
from GANDLF.utils import logger_setup


def _split_csv(input_csv: str, output_dir: str, config_path: Optional[str]):
    input_csv = os.path.normpath(input_csv)
    output_dir = os.path.normpath(output_dir)
    # initialize default
    config = yaml.safe_load(open(config_path, "r"))

    print("Config used for split:", config)

    split_data_and_save_csvs(input_csv, output_dir, config)

    print("Finished successfully.")


@click.command()
@click.option(
    "--input-csv",
    "-i",
    required=True,
    type=click.Path(exists=True, file_okay=True, dir_okay=False),
    help="Input CSV file which contains the data to be split.",
)
@click.option(
    "--output-dir",
    "-o",
    required=True,
    type=click.Path(exists=True, file_okay=False, dir_okay=True),
    help="Output directory to save the split data.",
)
@click.option(
    "--config",
    "-c",
    required=True,
    help="The GaNDLF config (in YAML) with the `nested_training` key specified to the folds needed.",
    type=click.Path(exists=True, file_okay=True, dir_okay=False),
)
@click.option(
    "--log-file",
    type=click.Path(),
    default=None,
    help="Output file which will contain the logs.",
)
@append_copyright_to_help
def new_way(input_csv: str, output_dir: str, log_file: str, config: Optional[str]):
    """Split the data into training, validation, and testing sets and save them as csvs in the output directory."""

    logger_setup(log_file)
    _split_csv(input_csv, output_dir, config)


# old-fashioned way of running gandlf via `gandlf_splitCSV`.
@deprecated(
    "This is a deprecated way of running GanDLF. Please, use `gandlf split-csv` cli command "
    + "instead of `gandlf_splitCSV`. Note that in new CLI tool some params were renamed:\n"
    + "  --inputCSV to --input-csv\n"
    + "  --outputDir to --output-dir\n"
    + "`gandlf_splitCSV` script would be deprecated soon."
)
def old_way():
    logger_setup()
    parser = argparse.ArgumentParser(
        prog="GANDLF_SplitCSV",
        formatter_class=argparse.RawTextHelpFormatter,
        description="Split the data into training, validation, and testing sets and save them as csvs in the output directory.\n\n"
        + copyrightMessage,
    )
    parser.add_argument(
        "-i",
        "--inputCSV",
        metavar="",
        default=None,
        type=str,
        required=True,
        help="Input CSV file which contains the data to be split.",
    )
    parser.add_argument(
        "-c",
        "--config",
        metavar="",
        default=None,
        required=True,
        type=str,
        help="The GaNDLF config (in YAML) with the `nested_training` key specified to the folds needed.",
    )
    parser.add_argument(
        "-o",
        "--outputDir",
        metavar="",
        default=None,
        type=str,
        required=True,
        help="Output directory to save the split data.",
    )

    args = parser.parse_args()

    # check for required parameters - this is needed here to keep the cli clean
    for param_name in ["inputCSV", "outputDir", "config"]:
        param_none_check = getattr(args, param_name)
        assert param_none_check is not None, f"Missing required parameter: {param_name}"

    _split_csv(
        input_csv=args.inputCSV, output_dir=args.outputDir, config_path=args.config
    )


# main function
if __name__ == "__main__":
    old_way()
